@@ -17,10 +17,18 @@
 
 #include <ldns/ldns.h>
 
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
+#endif
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
+#endif
+#ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
+#endif
 
 ldns_rdf *
 ldns_dname_cat_clone(const ldns_rdf *rd1, const ldns_rdf *rd2)
@@ -116,7 +124,7 @@ ldns_dname_reverse(const ldns_rdf *d)
 }
 
 ldns_rdf *
-ldns_dname_left_chop(ldns_rdf *d)
+ldns_dname_left_chop(const ldns_rdf *d)
 {
 	uint8_t label_pos;
 	ldns_rdf *chop;
@@ -427,8 +435,12 @@ ldns_dname_label(const ldns_rdf *rdf, uint8_t labelpos)
 	while ((len > 0) && src_pos < s) {
 		if (labelcnt == labelpos) {
 			/* found our label */
-			tmpnew = ldns_rdf_new_frm_data(LDNS_RDF_TYPE_DNAME, len + 1,
-					(ldns_rdf_data(rdf) + src_pos));
+			tmpnew = LDNS_MALLOC(ldns_rdf);
+			tmpnew->_type = LDNS_RDF_TYPE_DNAME;
+			tmpnew->_data = LDNS_XMALLOC(uint8_t, len + 2);
+			memset(tmpnew->_data, 0, len + 2);
+			memcpy(tmpnew->_data, ldns_rdf_data(rdf) + src_pos, len + 1);
+			tmpnew->_size = len + 2;
 			return tmpnew;
 		}
 		src_pos++;
