@@ -1,7 +1,7 @@
 /*
  * drill.c
  * the main file of drill
- * (c) 2005,2006 NLnet Labs
+ * (c) 2005-2008 NLnet Labs
  *
  * See the file LICENSE for the license
  *
@@ -10,6 +10,10 @@
 #include "drill.h"
 #include <ldns/ldns.h>
 
+#ifdef HAVE_SSL
+#include <openssl/err.h>
+#endif
+
 #define IP6_ARPA_MAX_LEN 65
 
 /* query debug, 2 hex dumps */
@@ -27,7 +31,7 @@ usage(FILE *stream, const char *progname)
 	fprintf(stream, "\t-D\t\tenable DNSSEC (DO bit)\n");
 	fprintf(stream, "\t-T\t\ttrace from the root down to <name>\n");
 	fprintf(stream, "\t-S\t\tchase signature(s) from <name> to a know key [*]\n");
-	fprintf(stream, "\t-V\t\tverbose mode (once shows question, twice for hexdumps)\n");
+	fprintf(stream, "\t-V <number>\tverbosity (0-5)\n");
 	fprintf(stream, "\t-Q\t\tquiet mode (overrules -V)\n");
 	fprintf(stream, "\n");
 	fprintf(stream, "\t-f file\t\tread packet from file and send it\n");
@@ -72,7 +76,7 @@ version(FILE *stream, const char *progname)
 {
         fprintf(stream, "%s version %s (ldns version %s)\n", progname, DRILL_VERSION, ldns_version());
         fprintf(stream, "Written by NLnet Labs.\n");
-        fprintf(stream, "\nCopyright (c) 2004-2006 NLnet Labs.\n");
+        fprintf(stream, "\nCopyright (c) 2004-2008 NLnet Labs.\n");
         fprintf(stream, "Licensed under the revised BSD license.\n");
         fprintf(stream, "There is NO warranty; not even for MERCHANTABILITY or FITNESS\n");
         fprintf(stream, "FOR A PARTICULAR PURPOSE.\n");
@@ -149,7 +153,7 @@ main(int argc, char *argv[])
 	PURPOSE = DRILL_QUERY;
 	qflags = LDNS_RD;
 	qport = LDNS_PORT;
-	verbosity = 0;
+	verbosity = 2;
 	qdnssec = false;
 	qfamily = LDNS_RESOLV_INETANY;
 	qfail = false;
@@ -590,16 +594,18 @@ main(int argc, char *argv[])
 				error("%s", "error pkt sending");
 				result = EXIT_FAILURE;
 			} else {
-				if (verbosity != -1) {
+				if (verbosity >= 3) {
 					ldns_pkt_print(stdout, pkt);
 				}
 				
 				if (!ldns_pkt_answer(pkt)) {
 					mesg("No answer in packet");
 				} else {
+					ldns_resolver_set_dnssec_anchors(res, ldns_rr_list_clone(key_list));
 					result = do_chase(res, qname, type,
 					                  clas, key_list, 
-					                  pkt, qflags, NULL);
+					                  pkt, qflags, NULL,
+								   verbosity);
 					if (result == LDNS_STATUS_OK) {
 						if (verbosity != -1) {
 							mesg("Chase successful");
@@ -740,6 +746,7 @@ main(int argc, char *argv[])
 				query_buffer = read_hex_buffer(query_file);
 				if (query_buffer) {
 					status = ldns_send_buffer(&pkt, res, query_buffer, NULL);
+					ldns_buffer_free(query_buffer);
 					if (status != LDNS_STATUS_OK) {
 						printf("Error: %s\n", ldns_get_errorstr_by_id(status));
 						exit(1);
@@ -870,5 +877,13 @@ main(int argc, char *argv[])
 */
 	xfree(tsig_data);
 	xfree(tsig_algorithm);
+
+#ifdef HAVE_SSL
+	ERR_remove_state(0);
+	CRYPTO_cleanup_all_ex_data();
+	ERR_free_strings();
+	EVP_cleanup();
+#endif
+
 	return result;
 }
