@@ -15,10 +15,18 @@
 
 #include <ldns/ldns.h>
 
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
+#endif
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
+#endif
+#ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
+#endif
 #include <sys/time.h>
 #include <errno.h>
 
@@ -31,7 +39,7 @@ ldns_send(ldns_pkt **result_packet, ldns_resolver *r, const ldns_pkt *query_pkt)
 
 	qb = ldns_buffer_new(LDNS_MIN_BUFLEN);
 
-	if (ldns_pkt_tsig(query_pkt)) {
+	if (query_pkt && ldns_pkt_tsig(query_pkt)) {
 		tsig_mac = ldns_rr_rdf(ldns_pkt_tsig(query_pkt), 3);
 	}
 
@@ -56,7 +64,7 @@ ldns_send_buffer(ldns_pkt **result, ldns_resolver *r, ldns_buffer *qb, ldns_rdf
 	struct sockaddr_storage *ns;
 	size_t ns_len;
 	struct timeval tv_s;
-        struct timeval tv_e;
+	struct timeval tv_e;
 
 	ldns_rdf **ns_array;
 	size_t *rtt;
@@ -67,7 +75,7 @@ ldns_send_buffer(ldns_pkt **result, ldns_resolver *r, ldns_buffer *qb, ldns_rdf
 	uint8_t *reply_bytes = NULL;
 	size_t reply_size = 0;
 	ldns_status status, send_status;
-	
+
 	assert(r != NULL);
 
 	status = LDNS_STATUS_OK;
@@ -84,7 +92,6 @@ ldns_send_buffer(ldns_pkt **result, ldns_resolver *r, ldns_buffer *qb, ldns_rdf
 
 	/* loop through all defined nameservers */
 	for (i = 0; i < ldns_resolver_nameserver_count(r); i++) {
-
 		if (rtt[i] == LDNS_RESOLV_RTT_INF) {
 			/* not reachable nameserver! */
 			continue;
@@ -112,6 +119,7 @@ ldns_send_buffer(ldns_pkt **result, ldns_resolver *r, ldns_buffer *qb, ldns_rdf
 		gettimeofday(&tv_s, NULL);
 
 		send_status = LDNS_STATUS_ERR;
+
 		/* reply_bytes implicitly handles our error */
 		if (1 == ldns_resolver_usevc(r)) {
 			for (retries = ldns_resolver_retry(r); retries > 0; retries--) {
@@ -201,6 +209,7 @@ ldns_send_buffer(ldns_pkt **result, ldns_resolver *r, ldns_buffer *qb, ldns_rdf
 #else
 	(void)tsig_mac;
 #endif /* HAVE_SSL */
+	
 	LDNS_FREE(reply_bytes);
 	if (result) {
 		*result = reply;
@@ -263,7 +272,7 @@ ldns_udp_connect(const struct sockaddr_storage *to, struct timeval timeout)
 			== -1) {
                 return 0;
         }
-	if (setsockopt(sockfd, SOL_SOCKET, SO_RCVTIMEO, &timeout,
+	if (setsockopt(sockfd, SOL_SOCKET, SO_RCVTIMEO, (void*)&timeout,
 				(socklen_t)sizeof(timeout))) {
                 /* might fail, in that case, use default for now */
 		/*
@@ -286,7 +295,7 @@ ldns_tcp_connect(const struct sockaddr_storage *to, socklen_t tolen,
 		return 0;
 	}
 
-        if (setsockopt(sockfd, SOL_SOCKET, SO_RCVTIMEO, &timeout,
+        if (setsockopt(sockfd, SOL_SOCKET, SO_RCVTIMEO, (void*)&timeout,
                         (socklen_t) sizeof(timeout))) {
                 /*perror("setsockopt");*/
                 close(sockfd);
