@@ -171,6 +171,7 @@ ldns_rdf *
 ldns_rdf_new_frm_data(ldns_rdf_type type, size_t size, const void *data)
 {
 	ldns_rdf *rdf;
+
 	rdf = LDNS_MALLOC(ldns_rdf);
 	if (!rdf) {
 		return NULL;
@@ -186,18 +187,16 @@ ldns_rdf_new_frm_data(ldns_rdf_type type, size_t size, const void *data)
 	ldns_rdf_set_type(rdf, type);
 	ldns_rdf_set_size(rdf, size);
 	memcpy(rdf->_data, data, size);
+
 	return rdf;
 }
 
 ldns_rdf *
 ldns_rdf_clone(const ldns_rdf *rd)
 {
-	if (rd) {
-		return (ldns_rdf_new_frm_data( ldns_rdf_get_type(rd),
-			ldns_rdf_size(rd), ldns_rdf_data(rd)));
-	} else {
-		return NULL;
-	}
+	assert(rd != NULL);
+	return (ldns_rdf_new_frm_data( ldns_rdf_get_type(rd),
+		ldns_rdf_size(rd), ldns_rdf_data(rd)));
 }
 
 void
@@ -253,6 +252,9 @@ ldns_rdf_new_frm_str(ldns_rdf_type type, const char *str)
 	case LDNS_RDF_TYPE_B64:
 		status = ldns_str2rdf_b64(&rdf, str);
 		break;
+	case LDNS_RDF_TYPE_B32_EXT:
+		status = ldns_str2rdf_b32_ext(&rdf, str);
+		break;
 	case LDNS_RDF_TYPE_HEX:
 		status = ldns_str2rdf_hex(&rdf, str);
 		break;
@@ -295,6 +297,12 @@ ldns_rdf_new_frm_str(ldns_rdf_type type, const char *str)
 	case LDNS_RDF_TYPE_NSAP:
 		status = ldns_str2rdf_nsap(&rdf, str);
 		break;
+	case LDNS_RDF_TYPE_NSEC3_SALT:
+		status = ldns_str2rdf_nsec3_salt(&rdf, str);
+		break;
+	case LDNS_RDF_TYPE_NSEC3_NEXT_OWNER:
+		status = ldns_str2rdf_b32_ext(&rdf, str);
+		break;
 	case LDNS_RDF_TYPE_NONE:
 	default:
 		/* default default ??? */
@@ -525,7 +533,7 @@ ldns_rdf_compare(const ldns_rdf *rd1, const ldns_rdf *rd2)
 		return -1;
 	}
 	i1 = ldns_rdf_size(rd1);
-	i2 = ldns_rdf_size(rd1);
+	i2 = ldns_rdf_size(rd2);
 
 	if (i1 < i2) {
 		return -1;
