@@ -14,13 +14,18 @@
 
 #include <ldns/ldns.h>
 
-#include <sys/types.h>
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+#ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
+#endif
 #include <time.h>
 
 #include <errno.h>
+#ifdef HAVE_NETDB_H
 #include <netdb.h> 
+#endif
 
 #include <limits.h>
 #ifdef HAVE_SYS_PARAM_H
@@ -62,7 +67,7 @@ ldns_str2rdf_time(ldns_rdf **rd, const char *time)
 	memset(&tm, 0, sizeof(tm));
 
 	if (strlen(time) == 14 &&
-	    sscanf(time, "%4d%2d%2d%2d%2d%2d", &tm.tm_year, &tm.tm_mon, &tm.tm_mday, &tm.tm_hour, &tm.tm_min, &tm.tm_sec)
+	    sscanf(time, "%4d%2d%2d%2d%2d%2d", &tm.tm_year, &tm.tm_mon, &tm.tm_mday, &tm.tm_hour, &tm.tm_min, &tm.tm_sec) == 6
 	   ) {
 	   	tm.tm_year -= 1900;
 	   	tm.tm_mon--;
@@ -115,6 +120,44 @@ ldns_str2rdf_time(ldns_rdf **rd, const char *time)
 	return LDNS_STATUS_INVALID_TIME;
 }
 
+ldns_status
+ldns_str2rdf_nsec3_salt(ldns_rdf **rd, const char *salt_str)
+{
+	uint8_t salt_length;
+	uint8_t c;
+
+	uint8_t *salt;
+	uint8_t *data;
+
+	salt_length = (uint8_t) strlen(salt_str);
+	if (salt_length == 1 && salt_str[0] == '-') {
+		salt_length = 0;
+	} else if (salt_length % 2 != 0) {
+		return LDNS_STATUS_INVALID_HEX;
+	}
+	
+	salt = LDNS_XMALLOC(uint8_t, salt_length / 2);
+	for (c = 0; c < salt_length; c += 2) {
+		if (isxdigit(salt_str[c]) && isxdigit(salt_str[c+1])) {
+			salt[c/2] = (uint8_t) ldns_hexdigit_to_int(salt_str[c]) * 16 +
+					  ldns_hexdigit_to_int(salt_str[c+1]);
+		} else {
+			LDNS_FREE(salt);
+			return LDNS_STATUS_INVALID_HEX;
+		}
+	}
+	salt_length = salt_length / 2;
+	
+	data = LDNS_XMALLOC(uint8_t, 1 + salt_length);
+	data[0] = salt_length;
+	memcpy(&data[1], salt, salt_length);
+	*rd = ldns_rdf_new_frm_data(LDNS_RDF_TYPE_NSEC3_SALT, 1 + salt_length, data);
+	LDNS_FREE(data);
+	LDNS_FREE(salt);
+
+	return LDNS_STATUS_OK;
+}
+
 ldns_status
 ldns_str2rdf_period(ldns_rdf **rd,const char *period)
 {
@@ -448,6 +491,29 @@ ldns_str2rdf_b64(ldns_rdf **rd, const char *str)
 	return LDNS_STATUS_OK;
 }
 
+ldns_status
+ldns_str2rdf_b32_ext(ldns_rdf **rd, const char *str)
+{
+	uint8_t *buffer;
+	int i;
+	/* first byte contains length of actual b32 data */
+	uint8_t len = b32_pton_calculate_size(strlen(str));
+	buffer = LDNS_XMALLOC(uint8_t, len + 1);
+	buffer[0] = len;
+	
+	i = b32_pton_extended_hex((const char*)str, strlen(str), buffer + 1, 
+                                  b32_ntop_calculate_size(strlen(str)));
+	if (i < 0) {
+		return LDNS_STATUS_INVALID_B32_EXT;
+	} else {
+		*rd = ldns_rdf_new_frm_data(
+			LDNS_RDF_TYPE_B32_EXT, (uint16_t) i + 1, buffer);
+	}
+	LDNS_FREE(buffer);
+
+	return LDNS_STATUS_OK;
+}
+
 ldns_status
 ldns_str2rdf_hex(ldns_rdf **rd, const char *str)
 {
@@ -489,7 +555,7 @@ ldns_status
 ldns_str2rdf_nsec(ldns_rdf **rd, const char *str)
 {
 	const char *delimiters = "\n\t ";
-	char token[LDNS_MAX_RDFLEN];
+	char *token = LDNS_XMALLOC(char, LDNS_MAX_RDFLEN);
 	uint8_t *bitmap = LDNS_XMALLOC(uint8_t, 1);
 	uint16_t bm_len = 0;
 	ldns_buffer *str_buf;
@@ -552,6 +618,8 @@ ldns_str2rdf_nsec(ldns_rdf **rd, const char *str)
 	*rd = ldns_rdf_new_frm_data(LDNS_RDF_TYPE_NSEC, cur_data_size, data);
 	if(data)
 		LDNS_FREE(data);
+	if (token)
+		LDNS_FREE(token);
 	if(bitmap)
 		LDNS_FREE(bitmap);
 	ldns_buffer_free(str_buf);
@@ -933,8 +1001,12 @@ ldns_str2rdf_wks(ldns_rdf **rd, const char *str)
 	LDNS_FREE(bitmap);
 	LDNS_FREE(data);
 	free(proto_str);
+#ifdef HAVE_ENDSERVENT
 	endservent();
+#endif
+#ifdef HAVE_ENDPROTOENT
 	endprotoent();
+#endif
 	
 	return LDNS_STATUS_OK;
 }
