@@ -50,6 +50,18 @@ xprintf_rr(ldns_rr *rr)
 	}
 }
 
+void xprintf_hex(uint8_t *data, size_t len)
+{
+	size_t i;
+	for (i = 0; i < len; i++) {
+		if (i > 0 && i % 20 == 0) {
+			printf("\t; %u - %u\n", (unsigned int) i - 19, (unsigned int) i);
+		}
+		printf("%02x ", (unsigned int) data[i]);
+	}
+	printf("\n");
+}
+
 ldns_lookup_table *
 ldns_lookup_by_name(ldns_lookup_table *table, const char *name)
 {
@@ -154,6 +166,28 @@ ldns_int_to_hexdigit(int i)
 	}
 }
 
+int
+ldns_hexstring_to_data(uint8_t *data, const char *str)
+{
+	unsigned int i;
+
+	if (!str || !data) {
+		return -1;
+	}
+
+	if (strlen(str) % 2 != 0) {
+		return -2;
+	}
+
+	for (i = 0; i < strlen(str) / 2; i++) {
+		data[i] = 
+			16 *	ldns_hexdigit_to_int(str[i*2]) +
+			ldns_hexdigit_to_int(str[i*2 + 1]);
+	}
+
+	return (int) i;
+}
+
 const char *
 ldns_version(void)
 {
